@@ -6,6 +6,7 @@
 #' @param DV.train a numeric or factor vector with compatible dimensions to \code{(IVs.train)}.
 #' @param IVs.test a matrix or data frame of variables of numeric or factor data types with compatible dimensions to \code{(IVs.train)}.  If NULL, will use \code{(IVs.train)} as default.
 #' @param type \code{NULL} (default).  To perform a classification of discrete integer classes from factor target variable \code{(DV.train)} with a base category of 1, set to \code{(type = "CLASS")}, else for continuous \code{(DV.train)} set to \code{(type = NULL)}.
+#' @param inference logical; \code{FALSE} (default) For inferential tasks, otherwise \code{inference = FALSE} is faster for predictive tasks.
 #' @param depth options: (integer, NULL, "max"); \code{(depth = NULL)}(default) Specifies the \code{order} parameter in the \link{NNS.reg} routine, assigning a number of splits in the regressors, analogous to tree depth.
 #' @param learner.trials integer; 100 (default) Sets the number of trials to obtain an accuracy \code{threshold} level.  If the number of all possible feature combinations is less than selected value, the minimum of the two values will be used.
 #' @param epochs integer; \code{2*length(DV.train)} (default) Total number of feature combinations to run.
@@ -48,6 +49,7 @@ NNS.boost <- function(IVs.train,
                       DV.train,
                       IVs.test = NULL,
                       type = NULL,
+                      inference = FALSE,
                       depth = NULL,
                       learner.trials = 100,
                       epochs = NULL,
@@ -63,6 +65,8 @@ NNS.boost <- function(IVs.train,
                       feature.importance = TRUE,
                       status = TRUE){
 
+  if(sum(is.na(cbind(IVs.train,DV.train))) > 0) stop("You have some missing values, please address.")
+
   if(is.null(obj.fn)) stop("Please provide an objective function")
 
   if(balance && is.null(type)) warning("type = 'CLASS' selected due to balance = TRUE.")
@@ -135,7 +139,6 @@ NNS.boost <- function(IVs.train,
 
 
   ### Representative samples
-  if(!is.numeric(y) || (sum((as.numeric(y)%%1)==0)==length(y) && length(unique(y)) < sqrt(length(y)))){
       rep.x <- data.table::data.table(x)
 
       rep.x <- rep.x[,lapply(.SD, function(z) fivenum(as.numeric(z))), by = .(y)]
@@ -144,10 +147,7 @@ NNS.boost <- function(IVs.train,
       rep.x <- rep.x[,-1]
 
       rep.x <- as.data.frame(rep.x)
-  } else {
-      rep.x <- x
-      rep.y <- NULL
-  }
+
 
   n <- ncol(x)
 
@@ -188,7 +188,7 @@ NNS.boost <- function(IVs.train,
 
       new.index <- unlist(new.index)
 
-      if(!is.numeric(y) || (sum((as.numeric(y)%%1)==0)==length(y) && length(unique(y)) < sqrt(length(y)))){
+
           new.iv.train <- data.table::data.table(x[-new.index,])
           new.iv.train <- new.iv.train[,lapply(.SD, as.double)]
 
@@ -199,10 +199,7 @@ NNS.boost <- function(IVs.train,
 
           new.iv.train <- rbind(new.iv.train, data.matrix(x[-new.index,]))
           new.dv.train <- c(new.dv.train, y[-new.index])
-      } else {
-          new.iv.train <- data.matrix(x[-new.index,])
-          new.dv.train <- y[-new.index]
-      }
+
 
       colnames(new.iv.train) <- features
 
@@ -222,7 +219,7 @@ NNS.boost <- function(IVs.train,
                            dim.red.method = "equal",
                            plot = FALSE, residual.plot = FALSE, order = depth,
                            factor.2.dummy = FALSE,
-                           ncores = 1, type = type)$Point.est
+                           ncores = 1, type = type, inference = inference)$Point.est
 
       predicted[is.na(predicted)] <- mean(predicted, na.rm = TRUE)
 
@@ -304,7 +301,7 @@ NNS.boost <- function(IVs.train,
 
       new.index <- unlist(new.index)
 
-      if(!is.numeric(y) || (sum((as.numeric(y)%%1)==0)==length(y) && length(unique(y)) < sqrt(length(y)))){
+
           new.iv.train <- data.table::data.table(x[-new.index, ])
           new.iv.train <- new.iv.train[, lapply(.SD,as.double)]
 
@@ -315,10 +312,7 @@ NNS.boost <- function(IVs.train,
 
           new.iv.train <- rbind(new.iv.train, data.matrix(x[-new.index,]))
           new.dv.train <- c(new.dv.train, y[-new.index])
-      } else {
-          new.iv.train <- data.matrix(x[-new.index,])
-          new.dv.train <- y[-new.index]
-      }
+
 
       actual <- as.numeric(y[new.index])
       new.iv.test <- x[new.index,]
@@ -339,7 +333,7 @@ NNS.boost <- function(IVs.train,
                            new.dv.train, point.est = new.iv.test[, features],
                            dim.red.method = "equal",
                            plot = FALSE, residual.plot = FALSE, order = depth,
-                           factor.2.dummy = FALSE, ncores = 1, type = type)$Point.est
+                           factor.2.dummy = FALSE, ncores = 1, type = type, inference = inference)$Point.est
 
       predicted[is.na(predicted)] <- mean(predicted, na.rm = TRUE)
       # Do not predict a new unseen class
@@ -412,7 +406,7 @@ NNS.boost <- function(IVs.train,
                              order = depth, dim.red.method = "cor",
                              ncores = 1,
                              stack = FALSE, status = status,
-                             type = type, dist = dist, folds = 5)$stack
+                             type = type, inference = inference, dist = dist, folds = 5)$stack
 
       estimates[is.na(unlist(estimates))] <- ifelse(!is.null(type), mode_class(unlist(na.omit(estimates))), mode(unlist(na.omit(estimates))))
 
