@@ -23,11 +23,14 @@ NNS.copula <- function (x,
                         continuous = TRUE,
                         plot = FALSE,
                         independence.overlay = FALSE){
+
+    if(sum(is.na(x)) > 0) stop("You have some missing values, please address.")
+
     A <- x
     n <- ncol(A)
     l <- dim(A)[1]
 
-    if(any(class(A)=="tbl")) A <- as.data.frame(A)
+    if(any(class(A)==c("tbl", "data.table"))) A <- as.data.frame(A)
 
     if(is.null(colnames(A))){
         colnames.list <- list()
