@@ -23,7 +23,9 @@ NNS.norm <- function(A,
                      chart.type = NULL,
                      location = "topleft"){
 
-  if(any(class(A)=="tbl")) A <- as.data.frame(A)
+  if(sum(is.na(A)) > 0) stop("You have some missing values, please address.")
+
+  if(any(class(A)==c("tbl", "data.table"))) A <- as.data.frame(A)
 
   m  <- colMeans(A)
   m[m==0] <- 1e-10
